# Generated by roxygen2: do not edit by hand

S3method("[",depower)
S3method(plot,bnb)
S3method(plot,depower)
S3method(plot,log_lognormal_dependent_two_sample)
S3method(plot,log_lognormal_independent_two_sample)
S3method(plot,log_lognormal_mixed_two_sample)
S3method(plot,log_lognormal_one_sample)
S3method(plot,nb)
S3method(power,bnb)
S3method(power,default)
S3method(power,log_lognormal_dependent_two_sample)
S3method(power,log_lognormal_independent_two_sample)
S3method(power,log_lognormal_mixed_two_sample)
S3method(power,log_lognormal_one_sample)
S3method(power,nb)
S3method(sim_null,bnb)
S3method(sim_null,log_lognormal_dependent_two_sample)
S3method(sim_null,log_lognormal_independent_two_sample)
S3method(sim_null,log_lognormal_mixed_two_sample)
S3method(sim_null,log_lognormal_one_sample)
S3method(sim_null,nb)
export(add_power_ci)
export(add_power_pi)
export(asymptotic)
export(eval_power_ci)
export(eval_power_pi)
export(glm_nb)
export(glmm_bnb)
export(glmm_poisson)
export(lrt_bnb)
export(lrt_nb)
export(mle_bnb_alt)
export(mle_bnb_null)
export(mle_nb_alt)
export(mle_nb_null)
export(nll_bnb_alt)
export(nll_bnb_null)
export(nll_nb_alt)
export(nll_nb_null)
export(power)
export(sim_bnb)
export(sim_log_lognormal)
export(sim_nb)
export(simulated)
export(t_test_paired)
export(t_test_welch)
export(wald_test_bnb)
export(wald_test_nb)
importFrom(Rdpack,reprompt)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,bind_cols)
importFrom(dplyr,collect)
importFrom(dplyr,cross_join)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,label_both)
importFrom(ggplot2,labeller)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,vars)
importFrom(glmmTMB,glmmTMB)
importFrom(glmmTMB,nbinom2)
importFrom(multidplyr,cluster_library)
importFrom(multidplyr,new_cluster)
importFrom(multidplyr,partition)
importFrom(mvnfast,rmvn)
importFrom(scales,percent)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,dnbinom)
importFrom(stats,logLik)
importFrom(stats,nlm)
importFrom(stats,nlminb)
importFrom(stats,optim)
importFrom(stats,pchisq)
importFrom(stats,poisson)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,rgamma)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stats,setNames)
importFrom(stats,vcov)
importFrom(utils,combn)
